@@ -25,6 +25,7 @@ from ros_buildfarm.common import get_github_project_url
 from ros_buildfarm.common import get_node_label
 from ros_buildfarm.common \
     import get_repositories_and_script_generating_key_files
+from ros_buildfarm.common import get_xunit_publisher_types_and_patterns
 from ros_buildfarm.common import git_github_orgunit
 from ros_buildfarm.common import JobValidationError
 from ros_buildfarm.common import write_groovy_script_and_configs
@@ -32,6 +33,7 @@ from ros_buildfarm.config import get_distribution_file
 from ros_buildfarm.config import get_index as get_config_index
 from ros_buildfarm.config import get_source_build_files
 from ros_buildfarm.git import get_repository
+from ros_buildfarm.jenkins import JenkinsProxy
 from ros_buildfarm.templates import expand_template
 from rosdistro import get_distribution_cache
 from rosdistro import get_index
@@ -89,7 +91,7 @@ def configure_devel_jobs(
     if build_file.test_pull_requests_force is not False:
         views[pull_request_view_name] = configure_devel_view(
             jenkins, pull_request_view_name, dry_run=dry_run)
-    if not jenkins:
+    if not isinstance(jenkins, JenkinsProxy):
         view_configs.update(views)
     groovy_data = {
         'dry_run': dry_run,
@@ -158,6 +160,30 @@ def configure_devel_jobs(
                 print("Pull request job for repository '%s'" % repo_name)
                 job_types.append('pull_request')
 
+        # check for abi support
+        run_abichecker = False
+        if build_file.test_abi_force is False:
+            pass
+        elif getattr(repo.source_repository, 'test_abi', None) is False:
+            pass
+        elif getattr(repo.source_repository, 'test_abi', None) is None and \
+                not build_file.test_abi_default:
+            pass
+        else:
+            print("ABI checking enabled for repository '%s'" % repo_name)
+            run_abichecker = True
+
+        # check for gpu support
+        require_gpu_support = False
+        if getattr(repo.source_repository, 'tests_require_gpu', None) is False:
+            pass
+        elif getattr(repo.source_repository, 'tests_require_gpu', None) is None and \
+                not build_file.tests_require_gpu_default:
+            pass
+        else:
+            print("GPU support is required for repository '%s'" % repo_name)
+            require_gpu_support = True
+
         for job_type in job_types:
             pull_request = job_type == 'pull_request'
             for os_name, os_code_name, arch in targets:
@@ -170,7 +196,9 @@ def configure_devel_jobs(
                         dist_cache=dist_cache, jenkins=jenkins, views=views,
                         is_disabled=is_disabled,
                         groovy_script=groovy_script,
-                        dry_run=dry_run)
+                        dry_run=dry_run,
+                        run_abichecker=run_abichecker,
+                        require_gpu_support=require_gpu_support)
                     if not pull_request:
                         devel_job_names.append(job_name)
                     else:
@@ -223,7 +251,9 @@ def configure_devel_job(
         groovy_script=None,
         source_repository=None,
         build_targets=None,
-        dry_run=False):
+        dry_run=False,
+        run_abichecker=None,
+        require_gpu_support=None):
     """
     Configure a single Jenkins devel job.
 
@@ -302,7 +332,8 @@ def configure_devel_job(
         index, config, rosdistro_name, source_build_name,
         build_file, os_name, os_code_name, arch, source_repository,
         repo_name, pull_request, job_name, dist_cache=dist_cache,
-        is_disabled=is_disabled)
+        is_disabled=is_disabled, run_abichecker=run_abichecker,
+        require_gpu_support=require_gpu_support)
     # jenkinsapi.jenkins.Jenkins evaluates to false if job count is zero
     if isinstance(jenkins, object) and jenkins is not False:
         from ros_buildfarm.jenkins import configure_job
@@ -322,7 +353,8 @@ def _get_devel_job_config(
         index, config, rosdistro_name, source_build_name,
         build_file, os_name, os_code_name, arch, source_repo_spec,
         repo_name, pull_request, job_name, dist_cache=None,
-        is_disabled=False):
+        is_disabled=False, run_abichecker=None,
+        require_gpu_support=None):
     template_name = 'devel/devel_job.xml.em'
 
     repository_args, script_generating_key_files = \
@@ -332,7 +364,7 @@ def _get_devel_job_config(
     if build_file.build_environment_variables:
         build_environment_variables = [
             '%s=%s' % (var, value)
-            for var, value in build_file.build_environment_variables.items()]
+            for var, value in sorted(build_file.build_environment_variables.items())]
 
     maintainer_emails = set([])
     if build_file.notify_maintainers and dist_cache and repo_name and \
@@ -390,9 +422,13 @@ def _get_devel_job_config(
         'arch': arch,
         'repository_args': repository_args,
         'build_tool': build_file.build_tool,
+        'build_tool_args': build_file.build_tool_args,
+        'build_tool_test_args': build_file.build_tool_test_args,
         'ros_version': ros_version,
         'build_environment_variables': build_environment_variables,
 
+        'run_abichecker': run_abichecker,
+        'require_gpu_support': require_gpu_support,
         'notify_compiler_warnings': build_file.notify_compiler_warnings,
         'notify_emails': build_file.notify_emails,
         'maintainer_emails': maintainer_emails,
@@ -402,9 +438,19 @@ def _get_devel_job_config(
 
         'timeout_minutes': build_file.jenkins_job_timeout,
 
+        # only Ubuntu Focal has a new enough pytest version which generates
+        # JUnit compliant result files
+        'xunit_publisher_types': get_xunit_publisher_types_and_patterns(
+            ros_version, os_name == 'ubuntu' and os_code_name != 'bionic'),
+
         'git_ssh_credential_id': config.git_ssh_credential_id,
 
         'collate_test_stats': build_file.collate_test_stats,
+
+        'benchmark_patterns': build_file.benchmark_patterns,
+        'benchmark_schema': build_file.benchmark_schema,
+
+        'shared_ccache': build_file.shared_ccache,
     }
     job_config = expand_template(template_name, job_data)
     return job_config
