@@ -1,4 +1,4 @@
-# Copyright 2014-2016 Open Source Robotics Foundation, Inc.
+# Copyright 2014-2020 Open Source Robotics Foundation, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,11 +17,16 @@ from __future__ import print_function
 from collections import OrderedDict
 import sys
 
+from ros_buildfarm.common import filter_blocked_dependent_package_names
+from ros_buildfarm.common import filter_buildfile_packages_recursively
 from ros_buildfarm.common import get_binarydeb_job_name
-from ros_buildfarm.common import get_debian_package_name
 from ros_buildfarm.common import get_default_node_label
+from ros_buildfarm.common import get_direct_dependencies
 from ros_buildfarm.common import get_github_project_url
+from ros_buildfarm.common import get_implicitly_ignored_package_names
 from ros_buildfarm.common import get_node_label
+from ros_buildfarm.common import get_os_package_name
+from ros_buildfarm.common import get_package_manifests
 from ros_buildfarm.common import get_release_binary_view_name
 from ros_buildfarm.common import get_release_job_prefix
 from ros_buildfarm.common import get_release_source_view_name
@@ -31,14 +36,20 @@ from ros_buildfarm.common \
 from ros_buildfarm.common import get_sourcedeb_job_name
 from ros_buildfarm.common import get_system_architecture
 from ros_buildfarm.common import JobValidationError
+from ros_buildfarm.common import package_format_mapping
 from ros_buildfarm.common import write_groovy_script_and_configs
 from ros_buildfarm.config import get_distribution_file
 from ros_buildfarm.config import get_index as get_config_index
 from ros_buildfarm.config import get_release_build_files
 from ros_buildfarm.git import get_repository
+from ros_buildfarm.jenkins import JenkinsProxy
+from ros_buildfarm.package_repo import get_package_repo_data
 from ros_buildfarm.templates import expand_template
+from rosdistro import get_cached_distribution
 from rosdistro import get_distribution_cache
+from rosdistro import get_distribution_file as rosdistro_get_distribution_file
 from rosdistro import get_index
+from rosdistro import get_package_condition_context
 
 
 def configure_release_jobs(
@@ -75,8 +86,25 @@ def configure_release_jobs(
         return
 
     pkg_names = dist_file.release_packages.keys()
+    cached_pkgs = _get_and_parse_distribution_cache(
+        index, rosdistro_name, pkg_names,
+        include_test_deps=build_file.include_test_dependencies,
+        include_group_deps=build_file.include_group_dependencies,
+        disable_groups_workaround=build_file.include_group_dependencies)
     filtered_pkg_names = build_file.filter_packages(pkg_names)
-    explicitly_ignored_pkg_names = set(pkg_names) - set(filtered_pkg_names)
+    explicitly_ignored_without_recursion_pkg_names = \
+        set(pkg_names) & set(build_file.package_ignore_list)
+    explicitly_ignored_pkg_names = \
+        set(pkg_names) - set(filtered_pkg_names) - explicitly_ignored_without_recursion_pkg_names
+
+    # Get package names for which the release version is missing,
+    # indicating the release has been "disabled" in the distribution file.
+    upstream_disabled_pkg_names = set(
+        p for r in dist_file.repositories.values()
+        if r.release_repository and not r.release_repository.version
+        for p in r.release_repository.package_names)
+    explicitly_ignored_pkg_names |= upstream_disabled_pkg_names
+
     if explicitly_ignored_pkg_names:
         print(('The following packages are being %s because of ' +
                'white-/blacklisting:') %
@@ -84,27 +112,8 @@ def configure_release_jobs(
         for pkg_name in sorted(explicitly_ignored_pkg_names):
             print('  -', pkg_name)
 
-    dist_cache = get_distribution_cache(index, rosdistro_name)
-
-    if explicitly_ignored_pkg_names:
-        # get direct dependencies from distro cache for each package
-        direct_dependencies = {}
-        for pkg_name in pkg_names:
-            direct_dependencies[pkg_name] = _get_direct_dependencies(
-                pkg_name, dist_cache, pkg_names) or set([])
-
-        # find recursive downstream deps for all explicitly ignored packages
-        ignored_pkg_names = set(explicitly_ignored_pkg_names)
-        while True:
-            implicitly_ignored_pkg_names = _get_downstream_package_names(
-                ignored_pkg_names, direct_dependencies)
-            if implicitly_ignored_pkg_names - ignored_pkg_names:
-                ignored_pkg_names |= implicitly_ignored_pkg_names
-                continue
-            break
-        implicitly_ignored_pkg_names = \
-            ignored_pkg_names - explicitly_ignored_pkg_names
-
+        implicitly_ignored_pkg_names = get_implicitly_ignored_package_names(
+            cached_pkgs, explicitly_ignored_pkg_names)
         if implicitly_ignored_pkg_names:
             print(('The following packages are being %s because their ' +
                    'dependencies are being ignored:') % ('ignored'
@@ -114,6 +123,14 @@ def configure_release_jobs(
             filtered_pkg_names = \
                 set(filtered_pkg_names) - implicitly_ignored_pkg_names
 
+    if explicitly_ignored_without_recursion_pkg_names:
+        print(('The following packages are being %s because of ' +
+               'ignore-listing:') %
+              ('ignored' if build_file.skip_ignored_packages else 'disabled'))
+        for pkg_name in sorted(explicitly_ignored_without_recursion_pkg_names):
+            print('  -', pkg_name)
+        filtered_pkg_names.difference_update(explicitly_ignored_without_recursion_pkg_names)
+
     # all further configuration will be handled by either the Jenkins API
     # or by a generated groovy script
     jenkins = False
@@ -124,26 +141,29 @@ def configure_release_jobs(
     all_view_configs = {}
     all_job_configs = OrderedDict()
 
-    job_name, job_config = configure_import_package_job(
-        config_url, rosdistro_name, release_build_name,
-        config=config, build_file=build_file, jenkins=jenkins, dry_run=dry_run)
-    if not jenkins:
-        all_job_configs[job_name] = job_config
+    for os_name, _ in platforms:
+        if package_format_mapping[os_name] not in ('rpm',):
+            job_name, job_config = configure_import_package_job(
+                config_url, rosdistro_name, release_build_name,
+                config=config, build_file=build_file, jenkins=jenkins, dry_run=dry_run)
+            if not isinstance(jenkins, JenkinsProxy):
+                all_job_configs[job_name] = job_config
+            break
 
     job_name, job_config = configure_sync_packages_to_main_job(
         config_url, rosdistro_name, release_build_name,
         config=config, build_file=build_file, jenkins=jenkins, dry_run=dry_run)
-    if not jenkins:
+    if not isinstance(jenkins, JenkinsProxy):
         all_job_configs[job_name] = job_config
 
     for os_name, os_code_name in platforms:
         for arch in sorted(build_file.targets[os_name][os_code_name]):
             job_name, job_config = configure_sync_packages_to_testing_job(
                 config_url, rosdistro_name, release_build_name,
-                os_code_name, arch,
+                os_name, os_code_name, arch,
                 config=config, build_file=build_file, jenkins=jenkins,
                 dry_run=dry_run)
-            if not jenkins:
+            if not isinstance(jenkins, JenkinsProxy):
                 all_job_configs[job_name] = job_config
 
     targets = []
@@ -154,7 +174,7 @@ def configure_release_jobs(
     views = configure_release_views(
         jenkins, rosdistro_name, release_build_name, targets,
         dry_run=dry_run)
-    if not jenkins:
+    if not isinstance(jenkins, JenkinsProxy):
         all_view_configs.update(views)
     groovy_data = {
         'dry_run': dry_run,
@@ -162,18 +182,11 @@ def configure_release_jobs(
     }
 
     # binary jobs must be generated in topological order
-    from catkin_pkg.package import parse_package_string
     from ros_buildfarm.common import topological_order_packages
-    pkgs = {}
-    for pkg_name in pkg_names:
-        if pkg_name not in dist_cache.release_package_xmls:
-            print("Skipping package '%s': no released package.xml in cache" %
-                  (pkg_name), file=sys.stderr)
-            continue
-        pkg_xml = dist_cache.release_package_xmls[pkg_name]
-        pkg = parse_package_string(pkg_xml)
-        pkgs[pkg_name] = pkg
-    ordered_pkg_tuples = topological_order_packages(pkgs)
+    for pkg_name in set(pkg_names).difference(cached_pkgs.keys()):
+        print("Skipping package '%s': no released package.xml in cache" %
+              (pkg_name), file=sys.stderr)
+    ordered_pkg_tuples = topological_order_packages(cached_pkgs)
 
     other_build_files = [v for k, v in build_files.items() if k != release_build_name]
 
@@ -219,7 +232,7 @@ def configure_release_jobs(
                         pkg_name, os_name, os_code_name,
                         config=config, build_file=build_file,
                         index=index, dist_file=dist_file,
-                        dist_cache=dist_cache,
+                        cached_pkgs=cached_pkgs,
                         jenkins=jenkins, views=views,
                         generate_import_package_job=False,
                         generate_sync_packages_jobs=False,
@@ -333,12 +346,47 @@ def configure_release_jobs(
             view_configs=all_view_configs)
 
 
-def _get_downstream_package_names(pkg_names, dependencies):
-    downstream_pkg_names = set([])
-    for pkg_name, deps in dependencies.items():
-        if deps.intersection(pkg_names):
-            downstream_pkg_names.add(pkg_name)
-    return downstream_pkg_names
+def _get_and_parse_distribution_cache(
+    index, rosdistro_name, pkg_names, include_test_deps, include_group_deps,
+    disable_groups_workaround,
+):
+    from catkin_pkg.package import parse_package_string
+    from catkin_pkg.package import Dependency
+    dist_cache = get_distribution_cache(index, rosdistro_name)
+    pkg_names = set(['ros_workspace']).union(pkg_names)
+    cached_pkgs = {
+        pkg_name: parse_package_string(pkg_xml)
+        for pkg_name, pkg_xml in dist_cache.release_package_xmls.items()
+        if pkg_name in pkg_names
+    }
+
+    condition_context = get_package_condition_context(index, rosdistro_name)
+    if disable_groups_workaround:
+        condition_context['DISABLE_GROUPS_WORKAROUND'] = '1'
+    for pkg in cached_pkgs.values():
+        pkg.evaluate_conditions(condition_context)
+    for pkg in cached_pkgs.values():
+        for group_depend in pkg.group_depends:
+            if group_depend.evaluated_condition is not False:
+                group_depend.extract_group_members(cached_pkgs.values())
+
+    # for ROS 2 distributions bloom injects a dependency on ros_workspace
+    # into almost all packages (except its dependencies)
+    # therefore the same dependency needs to to be injected here
+    distribution_type = index.distributions[rosdistro_name].get(
+        'distribution_type')
+    if distribution_type == 'ros2' and 'ros_workspace' in cached_pkgs:
+        no_ros_workspace_dep = set(['ros_workspace']).union(
+            get_direct_dependencies(
+                'ros_workspace', cached_pkgs, pkg_names,
+                include_test_deps=include_test_deps,
+                include_group_deps=include_group_deps))
+
+        for pkg_name, pkg in cached_pkgs.items():
+            if pkg_name not in no_ros_workspace_dep:
+                pkg.exec_depends.append(Dependency('ros_workspace'))
+
+    return cached_pkgs
 
 
 # Configure a Jenkins release job which consists of
@@ -348,9 +396,9 @@ def configure_release_job(
         config_url, rosdistro_name, release_build_name,
         pkg_name, os_name, os_code_name,
         config=None, build_file=None,
-        index=None, dist_file=None, dist_cache=None,
+        index=None, dist_file=None, cached_pkgs=None,
         jenkins=None, views=None,
-        generate_import_package_job=True,
+        generate_import_package_job=None,
         generate_sync_packages_jobs=True,
         is_disabled=False, other_build_files_same_platform=None,
         groovy_script=None,
@@ -368,6 +416,8 @@ def configure_release_job(
     if build_file is None:
         build_files = get_release_build_files(config, rosdistro_name)
         build_file = build_files[release_build_name]
+    if generate_import_package_job is None:
+        generate_import_package_job = package_format_mapping[os_name] not in ('rpm',)
 
     if index is None:
         index = get_index(config.rosdistro_index_url)
@@ -408,10 +458,14 @@ def configure_release_job(
             'choose one of the following: ' +
             ', '.join(sorted(build_file.targets[os_name].keys())))
 
-    if dist_cache is None and \
+    if cached_pkgs is None and \
             (build_file.notify_maintainers or
              build_file.abi_incompatibility_assumed):
-        dist_cache = get_distribution_cache(index, rosdistro_name)
+        cached_pkgs = _get_and_parse_distribution_cache(
+            index, rosdistro_name, [pkg_name],
+            include_test_deps=build_file.include_test_dependencies,
+            include_group_deps=build_file.include_group_dependencies,
+            disable_groups_workaround=build_file.include_group_dependencies)
     if jenkins is None:
         from ros_buildfarm.jenkins import connect
         jenkins = connect(config.jenkins_url)
@@ -438,7 +492,7 @@ def configure_release_job(
         for arch in build_file.targets[os_name][os_code_name]:
             configure_sync_packages_to_testing_job(
                 config_url, rosdistro_name, release_build_name,
-                os_code_name, arch,
+                os_name, os_code_name, arch,
                 config=config, build_file=build_file, jenkins=jenkins,
                 dry_run=dry_run)
 
@@ -467,7 +521,7 @@ def configure_release_job(
     job_config = _get_sourcedeb_job_config(
         config_url, rosdistro_name, release_build_name,
         config, build_file, os_name, os_code_name,
-        pkg_name, repo_name, repo.release_repository, dist_cache=dist_cache,
+        pkg_name, repo_name, repo.release_repository, cached_pkgs=cached_pkgs,
         is_disabled=is_source_disabled,
         other_build_files_same_platform=other_build_files_same_platform)
     # jenkinsapi.jenkins.Jenkins evaluates to false if job count is zero
@@ -479,14 +533,17 @@ def configure_release_job(
 
     dependency_names = []
     if build_file.abi_incompatibility_assumed:
-        dependency_names = _get_direct_dependencies(
-            pkg_name, dist_cache, pkg_names)
+        dependency_names = get_direct_dependencies(
+            pkg_name, cached_pkgs, pkg_names,
+            include_test_deps=build_file.include_test_dependencies,
+            include_group_deps=build_file.include_group_dependencies)
         # if dependencies are not yet available in rosdistro cache
         # skip binary jobs
         if dependency_names is None:
             print(("Skipping binary jobs for package '%s' because it is not " +
                    "yet in the rosdistro cache") % pkg_name, file=sys.stderr)
             return source_job_names, binary_job_names, job_configs
+        dependency_names.difference_update(build_file.package_ignore_list)
 
     # binarydeb jobs
     for arch in build_file.targets[os_name][os_code_name]:
@@ -507,7 +564,7 @@ def configure_release_job(
             config_url, rosdistro_name, release_build_name,
             config, build_file, os_name, os_code_name, arch,
             pkg_name, repo_name, repo.release_repository,
-            dist_cache=dist_cache, upstream_job_names=upstream_job_names,
+            cached_pkgs=cached_pkgs, upstream_job_names=upstream_job_names,
             is_disabled=is_disabled)
         # jenkinsapi.jenkins.Jenkins evaluates to false if job count is zero
         if isinstance(jenkins, object) and jenkins is not False:
@@ -540,33 +597,13 @@ def configure_release_views(
     return views
 
 
-def _get_direct_dependencies(pkg_name, dist_cache, pkg_names):
-    from catkin_pkg.package import parse_package_string
-    if pkg_name not in dist_cache.release_package_xmls:
-        return None
-    pkg_xml = dist_cache.release_package_xmls[pkg_name]
-    pkg = parse_package_string(pkg_xml)
-    # test dependencies are treated as build dependencies by bloom
-    # so we need them here to ensure that all dependencies are available
-    # before starting a build
-    depends = set([
-        d.name for d in (
-            pkg.buildtool_depends +
-            pkg.build_depends +
-            pkg.buildtool_export_depends +
-            pkg.build_export_depends +
-            pkg.exec_depends +
-            pkg.test_depends)
-        if d.name in pkg_names])
-    return depends
-
-
 def _get_sourcedeb_job_config(
         config_url, rosdistro_name, release_build_name,
         config, build_file, os_name, os_code_name,
-        pkg_name, repo_name, release_repository, dist_cache=None,
+        pkg_name, repo_name, release_repository, cached_pkgs=None,
         is_disabled=False, other_build_files_same_platform=None):
-    template_name = 'release/sourcedeb_job.xml.em'
+    package_format = package_format_mapping[os_name]
+    template_name = 'release/%s/sourcepkg_job.xml.em' % package_format
 
     repository_args, script_generating_key_files = \
         get_repositories_and_script_generating_key_files(build_file=build_file)
@@ -595,7 +632,7 @@ def _get_sourcedeb_job_config(
                 if other_build_file.notify_maintainers:
                     notify_maintainers = True
 
-    maintainer_emails = _get_maintainer_emails(dist_cache, pkg_name) \
+    maintainer_emails = _get_maintainer_emails(cached_pkgs, pkg_name) \
         if notify_maintainers \
         else set([])
 
@@ -605,8 +642,8 @@ def _get_sourcedeb_job_config(
         'job_priority': build_file.jenkins_source_job_priority,
         'node_label': get_node_label(
             build_file.jenkins_source_job_label,
-            get_default_node_label('%s_%s' % (
-                rosdistro_name, 'sourcedeb'))),
+            get_default_node_label('%s_%s%s' % (
+                rosdistro_name, 'source', package_format))),
 
         'disabled': is_disabled,
 
@@ -625,8 +662,9 @@ def _get_sourcedeb_job_config(
 
         'sourcedeb_files': sourcedeb_files,
 
-        'import_package_job_name': get_import_package_job_name(rosdistro_name),
-        'debian_package_name': get_debian_package_name(
+        'import_package_job_name': get_import_package_job_name(
+            rosdistro_name, package_format),
+        'debian_package_name': get_os_package_name(
             rosdistro_name, pkg_name),
 
         'notify_emails': notify_emails,
@@ -635,6 +673,7 @@ def _get_sourcedeb_job_config(
 
         'timeout_minutes': build_file.jenkins_source_job_timeout,
 
+        'upload_host': build_file.upload_host,
         'credential_id': build_file.upload_credential_id,
 
         'git_ssh_credential_id': config.git_ssh_credential_id,
@@ -647,9 +686,10 @@ def _get_binarydeb_job_config(
         config_url, rosdistro_name, release_build_name,
         config, build_file, os_name, os_code_name, arch,
         pkg_name, repo_name, release_repository,
-        dist_cache=None, upstream_job_names=None,
+        cached_pkgs=None, upstream_job_names=None,
         is_disabled=False):
-    template_name = 'release/binarydeb_job.xml.em'
+    package_format = package_format_mapping[os_name]
+    template_name = 'release/%s/binarypkg_job.xml.em' % package_format
 
     repository_args, script_generating_key_files = \
         get_repositories_and_script_generating_key_files(build_file=build_file)
@@ -666,12 +706,12 @@ def _get_binarydeb_job_config(
     if build_file.build_environment_variables:
         build_environment_variables = [
             '%s=%s' % (var, value)
-            for var, value in build_file.build_environment_variables.items()]
+            for var, value in sorted(build_file.build_environment_variables.items())]
 
     sync_to_testing_job_name = [get_sync_packages_to_testing_job_name(
-        rosdistro_name, os_code_name, arch)]
+        rosdistro_name, os_name, os_code_name, arch)]
 
-    maintainer_emails = _get_maintainer_emails(dist_cache, pkg_name) \
+    maintainer_emails = _get_maintainer_emails(cached_pkgs, pkg_name) \
         if build_file.notify_maintainers \
         else set([])
 
@@ -679,10 +719,11 @@ def _get_binarydeb_job_config(
         'github_url': get_github_project_url(release_repository.url),
 
         'job_priority': build_file.jenkins_binary_job_priority,
+        'job_weight': build_file.jenkins_binary_job_weight_overrides.get(pkg_name),
         'node_label': get_node_label(
             build_file.jenkins_binary_job_label,
-            get_default_node_label('%s_%s_%s' % (
-                rosdistro_name, 'binarydeb', release_build_name))),
+            get_default_node_label('%s_%s%s_%s' % (
+                rosdistro_name, 'binary', package_format, release_build_name))),
 
         'disabled': is_disabled,
 
@@ -706,8 +747,9 @@ def _get_binarydeb_job_config(
         'binarydeb_files': binarydeb_files,
         'build_environment_variables': build_environment_variables,
 
-        'import_package_job_name': get_import_package_job_name(rosdistro_name),
-        'debian_package_name': get_debian_package_name(
+        'import_package_job_name': get_import_package_job_name(
+            rosdistro_name, package_format),
+        'debian_package_name': get_os_package_name(
             rosdistro_name, pkg_name),
 
         'child_projects': sync_to_testing_job_name,
@@ -715,10 +757,14 @@ def _get_binarydeb_job_config(
         'notify_emails': build_file.notify_emails,
         'maintainer_emails': maintainer_emails,
         'notify_maintainers': build_file.notify_maintainers,
+        'skip_tests': not build_file.run_package_tests,
 
         'timeout_minutes': build_file.jenkins_binary_job_timeout,
 
+        'upload_host': build_file.upload_host,
         'credential_id': build_file.upload_credential_id,
+
+        'shared_ccache': build_file.shared_ccache,
     }
     job_config = expand_template(template_name, job_data)
     return job_config
@@ -736,8 +782,13 @@ def configure_import_package_job(
         from ros_buildfarm.jenkins import connect
         jenkins = connect(config.jenkins_url)
 
-    job_name = get_import_package_job_name(rosdistro_name)
-    job_config = _get_import_package_job_config(build_file)
+    package_formats = set(
+        package_format_mapping[os_name] for os_name in build_file.targets.keys())
+    assert len(package_formats) == 1
+    package_format = package_formats.pop()
+
+    job_name = get_import_package_job_name(rosdistro_name, package_format)
+    job_config = _get_import_package_job_config(build_file, package_format)
 
     # jenkinsapi.jenkins.Jenkins evaluates to false if job count is zero
     if isinstance(jenkins, object) and jenkins is not False:
@@ -746,25 +797,27 @@ def configure_import_package_job(
     return (job_name, job_config)
 
 
-def get_import_package_job_name(rosdistro_name):
+def get_import_package_job_name(rosdistro_name, package_format):
     view_name = get_release_job_prefix(rosdistro_name)
-    return '%s_import-package' % view_name
+    return '%s_import-package%s' % (
+        view_name, '' if package_format == 'deb' else '-' + package_format)
 
 
-def _get_import_package_job_config(build_file):
-    template_name = 'release/import_package_job.xml.em'
+def _get_import_package_job_config(build_file, package_format):
+    template_name = 'release/%s/import_package_job.xml.em' % package_format
     job_data = {
         'target_queue': build_file.target_queue,
         'abi_incompatibility_assumed': build_file.abi_incompatibility_assumed,
         'notify_emails': build_file.notify_emails,
+        'ros_buildfarm_repository': get_repository(),
     }
     job_config = expand_template(template_name, job_data)
     return job_config
 
 
 def configure_sync_packages_to_testing_job(
-        config_url, rosdistro_name, release_build_name, os_code_name, arch,
-        config=None, build_file=None, jenkins=None, dry_run=False):
+        config_url, rosdistro_name, release_build_name, os_name, os_code_name,
+        arch, config=None, build_file=None, jenkins=None, dry_run=False):
     if config is None:
         config = get_config_index(config_url)
     if build_file is None:
@@ -775,10 +828,10 @@ def configure_sync_packages_to_testing_job(
         jenkins = connect(config.jenkins_url)
 
     job_name = get_sync_packages_to_testing_job_name(
-        rosdistro_name, os_code_name, arch)
+        rosdistro_name, os_name, os_code_name, arch)
     job_config = _get_sync_packages_to_testing_job_config(
-        config_url, rosdistro_name, release_build_name, os_code_name, arch,
-        config, build_file)
+        config_url, rosdistro_name, release_build_name, os_name, os_code_name,
+        arch, config, build_file)
 
     # jenkinsapi.jenkins.Jenkins evaluates to false if job count is zero
     if isinstance(jenkins, object) and jenkins is not False:
@@ -788,19 +841,21 @@ def configure_sync_packages_to_testing_job(
 
 
 def get_sync_packages_to_testing_job_name(
-        rosdistro_name, os_code_name, arch):
+        rosdistro_name, os_name, os_code_name, arch):
     view_name = get_release_job_prefix(rosdistro_name)
-    return '%s_sync-packages-to-testing_%s_%s' % \
-        (view_name, os_code_name, arch)
+    return '%s_sync-packages-to-testing%s_%s_%s' % (
+        view_name, '' if package_format_mapping[os_name] == 'deb' else '_' + os_name,
+        os_code_name, arch)
 
 
 def _get_sync_packages_to_testing_job_config(
-        config_url, rosdistro_name, release_build_name, os_code_name, arch,
-        config, build_file):
-    template_name = 'release/sync_packages_to_testing_job.xml.em'
+        config_url, rosdistro_name, release_build_name, os_name, os_code_name,
+        arch, config, build_file):
+    package_format = package_format_mapping[os_name]
+    template_name = 'release/%s/sync_packages_to_testing_job.xml.em' % package_format
 
     repository_args, script_generating_key_files = \
-        get_repositories_and_script_generating_key_files(build_file=build_file)
+        get_repositories_and_script_generating_key_files(config=config)
 
     job_data = {
         'ros_buildfarm_repository': get_repository(),
@@ -810,10 +865,14 @@ def _get_sync_packages_to_testing_job_config(
         'config_url': config_url,
         'rosdistro_name': rosdistro_name,
         'release_build_name': release_build_name,
+        'os_name': os_name,
         'os_code_name': os_code_name,
         'arch': arch,
         'repository_args': repository_args,
 
+        'import_package_job_name': get_import_package_job_name(
+            rosdistro_name, package_format),
+
         'notify_emails': build_file.notify_emails,
     }
     job_config = expand_template(template_name, job_data)
@@ -832,10 +891,15 @@ def configure_sync_packages_to_main_job(
         from ros_buildfarm.jenkins import connect
         jenkins = connect(config.jenkins_url)
 
+    package_formats = set(
+        package_format_mapping[os_name] for os_name in build_file.targets.keys())
+    assert len(package_formats) == 1
+    package_format = package_formats.pop()
+
     job_name = get_sync_packages_to_main_job_name(
-        rosdistro_name)
+        rosdistro_name, package_format)
     job_config = _get_sync_packages_to_main_job_config(
-        rosdistro_name, build_file)
+        rosdistro_name, build_file, package_format)
 
     # jenkinsapi.jenkins.Jenkins evaluates to false if job count is zero
     if isinstance(jenkins, object) and jenkins is not False:
@@ -844,29 +908,82 @@ def configure_sync_packages_to_main_job(
     return (job_name, job_config)
 
 
-def get_sync_packages_to_main_job_name(rosdistro_name):
+def get_sync_packages_to_main_job_name(rosdistro_name, package_format):
     view_name = get_release_job_prefix(rosdistro_name)
-    return '%s_sync-packages-to-main' % view_name
+    return '%s_sync-packages-to-main%s' % (
+        view_name, '' if package_format == 'deb' else '-' + package_format)
 
 
-def _get_sync_packages_to_main_job_config(rosdistro_name, build_file):
-    template_name = 'release/sync_packages_to_main_job.xml.em'
+def _get_sync_packages_to_main_job_config(rosdistro_name, build_file, package_format):
+    template_name = 'release/%s/sync_packages_to_main_job.xml.em' % package_format
     job_data = {
+        'ros_buildfarm_repository': get_repository(),
         'rosdistro_name': rosdistro_name,
 
+        'deb_sync_to_main_job_name': get_sync_packages_to_main_job_name(rosdistro_name, 'deb'),
+        'sync_targets': build_file.targets,
+
         'notify_emails': build_file.notify_emails,
     }
     job_config = expand_template(template_name, job_data)
     return job_config
 
 
-def _get_maintainer_emails(dist_cache, pkg_name):
+def _get_maintainer_emails(cached_pkgs, pkg_name):
     maintainer_emails = set([])
     # add maintainers listed in latest release to recipients
-    if dist_cache and pkg_name in dist_cache.release_package_xmls:
-        from catkin_pkg.package import parse_package_string
-        pkg_xml = dist_cache.release_package_xmls[pkg_name]
-        pkg = parse_package_string(pkg_xml)
-        for m in pkg.maintainers:
+    if cached_pkgs and pkg_name in cached_pkgs:
+        for m in cached_pkgs[pkg_name].maintainers:
             maintainer_emails.add(m.email)
     return maintainer_emails
+
+
+def partition_packages(
+        config_url, rosdistro_name, release_build_name, target, cache_dir,
+        deduplicate_dependencies=False, dist_cache=None):
+    """Check all packages in the rosdistro and compare to the debian packages repository.
+
+    Return the set of all packages and the set of missing ones.
+    """
+    # fetch debian package list
+    config = get_config_index(config_url)
+    index = get_index(config.rosdistro_index_url)
+    dist_file = rosdistro_get_distribution_file(index, rosdistro_name)
+    build_files = get_release_build_files(config, rosdistro_name)
+    build_file = build_files[release_build_name]
+
+    # Check that apt repos status
+    repo_index = get_package_repo_data(
+        build_file.target_repository, [target], cache_dir)[target]
+
+    # for each release package which matches the release build file
+    # check if a binary package exists
+    binary_packages = set()
+    all_pkg_names = dist_file.release_packages.keys()
+
+    # Remove packages without versions declared.
+    def get_package_version(dist_file, pkg_name):
+        pkg = dist_file.release_packages[pkg_name]
+        repo_name = pkg.repository_name
+        repo = dist_file.repositories[repo_name]
+        return repo.release_repository.version
+    all_pkg_names = [p for p in all_pkg_names if get_package_version(dist_file, p)]
+
+    distribution = get_cached_distribution(index, rosdistro_name, cache=dist_cache)
+    pkg_names = filter_buildfile_packages_recursively(all_pkg_names, build_file, distribution)
+    for pkg_name in sorted(pkg_names):
+        debian_pkg_name = get_os_package_name(rosdistro_name, pkg_name)
+        if debian_pkg_name in repo_index:
+            binary_packages.add(pkg_name)
+
+    # check that all elements from whitelist are present
+    missing_binary_packages = set(pkg_names) - binary_packages
+
+    if deduplicate_dependencies:
+        # Do not list missing packages that are dependencies of other missing ones
+        cached_pkgs = get_package_manifests(distribution)
+        missing_binary_packages = filter_blocked_dependent_package_names(
+            cached_pkgs,
+            missing_binary_packages)
+
+    return binary_packages, missing_binary_packages
